/*
 * $Id: ConferenceStatus.cpp,v 1.3.2.5 2005/08/25 07:54:05 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * For a license to use the sems software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ConferenceStatus.h"
#include "Conference.h"
#include "MultiPartyMixerSocket.h"
#include "MultiPartyMixer.h"

#include "AmAudio.h"
#include "log.h"

#include <assert.h>
#include <unistd.h>

DialogEvent::DialogEvent(int action, ConferenceDialog* dialog)
    : AmEvent(action), dialog(dialog)
{
}

ConferenceStatus::ConferenceStatus(const string& conference_id)
    : conf_id(conference_id), request_q(this), mixer(&str_set)
{
    if(!ConferenceFactory::TracePath.empty())
	str_set.enableRTPTrace(ConferenceFactory::TracePath + "/" + conf_id);
}

ConferenceStatus::~ConferenceStatus()
{
}

void ConferenceStatus::run()
{
    bool had_lonely_user;
    ConferenceDialog* lonely_dialog = 0;

    runcond.set(true);
    DBG("ConferenceStatus running...\n");

    while (runcond.get()) {
	
	had_lonely_user = (dialogs.size() == 1);
 	request_q.processEvents();
	
 	if (dialogs.empty()){
 	    DBG("no more users: terminating %s\n",conf_id.c_str());
 	    runcond.set(false);
 	}

	if (!had_lonely_user && (dialogs.size() == 1)){

 	    AmAudioFile* announcement_file = new AmAudioFile("Wav");
 	    announcement_file->loop.set(true);
 	    if(!announcement_file->open(ConferenceFactory::LonelyUserFile.c_str(),
					AmAudioFile::Read)) {

		lonely_dialog = *dialogs.begin();
		assert(lonely_dialog);
		lonely_dialog->playSock->addToPlaylist(announcement_file,
						       MultiPartyMixerSocket::User,
						       MultiPartyMixerSocket::Q_Back);
	    }
	}

	if (had_lonely_user && (dialogs.size() != 1)){

	    assert(lonely_dialog);
	    lonely_dialog->playSock->clearPlaylist(MultiPartyMixerSocket::User);

	    if(dialogs.size() > 1){

		AmAudioFile* beep_file = new AmAudioFile("Wav");
		if(!beep_file->open(ConferenceFactory::JoinSound.c_str(),
				    AmAudioFile::Read)) {
		
		    lonely_dialog->playSock->addToPlaylist(beep_file,
							   MultiPartyMixerSocket::User,
							   MultiPartyMixerSocket::Q_Front);
		}
	    }

	    lonely_dialog = 0;
	}

 	if (runcond.get()) {
 	    if (str_set.exchange() < 0){
 		runcond.set(false);
 		ERROR("while send & receive...\n");
 	    }
 	}
    }
    
    if(!dialogs.empty()) {
  	for( set<ConferenceDialog*>::iterator it = dialogs.begin();
  	     it != dialogs.end(); ++it ) {
	    
  	    AmSession* sess = (*it)->getSession();
	    
  	    sess->req->bye();
  	    sess->destroy();
  	}

 	while(!dialogs.empty()){
 	    DBG("!dialogs.empty(): waiting 1 second...\n");
 	    sleep(1);
 	    request_q.processEvents();
 	}
    }
    
    DBG("ConferenceStatus finished...\n");
    ConferenceStatusContainer::instance()->destroyStatus(conf_id);
}

void ConferenceStatus::on_stop()
{
    DBG("ConferenceStatus::on_stop()\n");
    runcond.set(false);
}

void ConferenceStatus::process(AmEvent* event)
{
    ConferenceStatusCmd* cmd = dynamic_cast<ConferenceStatusCmd*>(event);
    if(!cmd){
	ERROR("ConferenceStatus: invalid event received.\n");
	return;
    }

    switch(cmd->event_id){

    case CSC_Insert:
	{
	    int channel = mixer.addChannel();
	    ConferenceDialog* dialog = cmd->dialog;


	    dialog->channel_id = channel;
	    dialogs.insert(dialog);

	    // plug mixer output into input
	    dialog->playSock.reset(new MultiPartyMixerSocket( &mixer,
							      MultiPartyMixerSocket::Play,
							      channel));

	    dialog->recSock.reset(new MultiPartyMixerSocket( &mixer,
							     MultiPartyMixerSocket::Record,
							     channel));

	    AmRtpStreamInfo str_info( AmRtpStreamInfo::ST_Duplex,
				      dialog->playSock.get(),dialog->recSock.get());

	    str_set.addStream( &dialog->getSession()->rtp_str, str_info );

	    if(dialogs.size() > 2){

		for(set<ConferenceDialog*>::iterator it = dialogs.begin(); 
		    it != dialogs.end(); ++it){
		    
		    if(*it == dialog)
			continue;
		    
		    AmAudioFile* beep_file = new AmAudioFile("Wav");
		    if(!beep_file->open(ConferenceFactory::JoinSound.c_str(),
					AmAudioFile::Read)) {
		    
			(*it)->playSock->addToPlaylist(beep_file,
						       MultiPartyMixerSocket::User,
						       MultiPartyMixerSocket::Q_Front);
		    }
		}
	    }
	}
	break;

    case CSC_Remove:
	{
	    set<ConferenceDialog*>::iterator it;
	    if( (it = dialogs.find(cmd->dialog)) != dialogs.end() ) {

		str_set.removeStream( &(*it)->getSession()->rtp_str );
		mixer.removeChannel( (*it)->channel_id );
		dialogs.erase(it);
		(*it)->unbind();
		DBG("dialog removed & unbound !\n");

 		for(it = dialogs.begin(); it != dialogs.end(); ++it){

 		    AmAudioFile* beep_file = new AmAudioFile("Wav");
 		    if(!beep_file->open(ConferenceFactory::DropSound.c_str(),
					AmAudioFile::Read)) {

			(*it)->playSock->addToPlaylist(beep_file,
						       MultiPartyMixerSocket::User,
						       MultiPartyMixerSocket::Q_Front);
		    }
 		}
	    }
	}
	break;
    }
}

void ConferenceStatus::pushRequest(Action a, ConferenceDialog* cd)
{
    request_q.postEvent(new ConferenceStatusCmd(int(a),cd));
}

void ConferenceStatus::addDialog(ConferenceDialog* dialog)
{
    pushRequest(CSC_Insert,dialog);
}

void ConferenceStatus::removeDialog(ConferenceDialog* dialog)
{
    pushRequest(CSC_Remove,dialog);
}

ConferenceStatusContainer* ConferenceStatusContainer::_instance=0;

ConferenceStatusContainer::ConferenceStatusContainer()
    : _run_cond(false)
{
}

ConferenceStatusContainer* ConferenceStatusContainer::instance()
{
    if(!_instance){
	_instance = new ConferenceStatusContainer();
	_instance->start();
    }

    return _instance;
}

bool ConferenceStatusContainer::createStatus(const string& conf_id)
{
    cid2s_mut.lock();

    ConferenceStatus* status=0;
    bool is_new;

    map<string,ConferenceStatus*>::iterator conf_it;
    if( ( conf_it = cid2status.find(conf_id) ) 
	!= cid2status.end() ) {

	status = conf_it->second;
	is_new = false;
    }
    else {

	DBG("creating new ConferenceStatus: conf_id=%s\n",conf_id.c_str());
	status = new ConferenceStatus(conf_id);
	is_new = true;

	cid2status.insert(std::make_pair(conf_id,status));
    }
    
    cid2s_mut.unlock();
    return is_new;
}

void ConferenceStatusContainer::destroyStatus(const string& conf_id)
{
    cid2s_mut.lock();
    map<string,ConferenceStatus*>::iterator conf_it;
    if( ( conf_it = cid2status.find(conf_id) ) 
	!= cid2status.end() ) {

	ConferenceStatus* status = conf_it->second;
	cid2status.erase(conf_it);
	
	ds_mut.lock();
	status->stop();
	d_status.push(status);
	ds_mut.unlock();
	_run_cond.set(true);
    }    
    cid2s_mut.unlock();
}

bool ConferenceStatusContainer::addDialog(const string& conf_id, ConferenceDialog* dialog)
{
    bool ret=false;
    cid2s_mut.lock();

    map<string,ConferenceStatus*>::iterator conf_it;
    if( ( conf_it = cid2status.find(conf_id) ) 
	!= cid2status.end() ) {

	ConferenceStatus* status = conf_it->second;
	status->addDialog(dialog);

	if(status->getStopped())
	    status->start();

	ret = true;
    }

    cid2s_mut.unlock();
    return ret;
}

void ConferenceStatusContainer::removeDialog(const string& conf_id, ConferenceDialog* dialog)
{
    cid2s_mut.lock();

    map<string,ConferenceStatus*>::iterator conf_it;
    if( ( conf_it = cid2status.find(conf_id) ) 
	!= cid2status.end() ) {

	ConferenceStatus* status = conf_it->second;
	status->removeDialog(dialog);
    }

    cid2s_mut.unlock();
}

void ConferenceStatusContainer::run()
{
    while(1){

	_run_cond.wait_for();

	// Let some time to Status to stop 
	sleep(5);

	ds_mut.lock();
	DBG("ConferenceStatus cleaner starting its work\n");

	try {
	    queue<ConferenceStatus*> n_status;

	    while(!d_status.empty()){

	        ConferenceStatus* cur_status = d_status.front();
		d_status.pop();

		ds_mut.unlock();

		if(cur_status->getStopped()){
		    delete cur_status;
		    DBG("status 0x%p has been destroyed'\n",cur_status);
		}
		else {
		    DBG("status 0x%p still running\n",cur_status);
		    n_status.push(cur_status);
		}

		ds_mut.lock();
	    }

	    swap(d_status,n_status);

	}catch(...){}

	bool more = !d_status.empty();
	ds_mut.unlock();

	DBG("ConferenceStatus cleaner finished\n");
	if(!more)
	    _run_cond.set(false);
    }
}

void ConferenceStatusContainer::on_stop()
{
}
