#include "StatsUDPServer.h"
#include "Statistics.h"
#include "SemsConfiguration.h"
#include "AmUtils.h"
#include "log.h"

#include <string>
using std::string;

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <errno.h>
#include <netinet/ip.h>

int msg_get_line(char*& msg_c, char* str, size_t len)
{
    size_t l;
    char*  s=str;

    if(!len)
	return 0;
    
    for(l=len; l && (*msg_c) && (*msg_c !='\n'); msg_c++ ){
	if(*msg_c!='\r'){
	    *(s++) = *msg_c;
	    l--;
	}
    }

    if(*msg_c)
	msg_c++;

    if(l>0){
	// We need one more character
	// for trailing '\0'.
	*s='\0';

	return int(s-str);
    }
    else
	// buffer overran.
	return -1;
}

int msg_get_param(char*& msg_c, string& p)
{
    char line_buf[MSG_BUF_SIZE];

    if( msg_get_line(msg_c,line_buf,MSG_BUF_SIZE) != -1 ){

	if(!strcmp(".",line_buf))
	    line_buf[0]='\0';

	p = line_buf;
	return 0;
    }

    return -1;
}

StatsUDPServer* StatsUDPServer::_instance=0;

StatsUDPServer* StatsUDPServer::instance()
{
    if(!_instance) {
	_instance = new StatsUDPServer();
	_instance->start();
    }
    return _instance;
}

StatsUDPServer::StatsUDPServer()
    : sd(0)
{
    sc = AmSessionContainer::instance();
}

StatsUDPServer::~StatsUDPServer()
{
    if(sd)
	close(sd);
}

void StatsUDPServer::run()
{
    SemsConfiguration m_config;
    char* p = 0;
    int udp_port = 0;
    int optval;

    if(m_config.reloadModuleConfig(MOD_NAME) != 1){
	WARN("could not load any config for module: %s\n",MOD_NAME);
    }
    else if( ((p = m_config.getValueForKey("monit_udp_port")) != NULL) && (*p != '\0') ){
 	if(sscanf(p,"%u",&udp_port) != 1)
	    ERROR("could not convert monit_udp_port ('%s') to an integer.\n",p);
    }
    else 
	WARN("no monit_udp_port specified in configuration\n");

    if(!udp_port){
	ERROR("using default monit_udp_port.\n");
	udp_port = DEFAULT_MONIT_UDP_PORT;
    }

    sd = socket(PF_INET,SOCK_DGRAM,0);
    if(sd == -1){
	ERROR("could not open socket: %s\n",strerror(errno));
	return;
    }

    /* set sock opts? */
    optval=1;
    if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, (void*)&optval, sizeof(optval)) ==-1){
        ERROR("ERROR: setsockopt(reuseaddr): %s\n", strerror(errno));
	return;	
    }
    /* tos */
    optval=IPTOS_LOWDELAY;
    if (setsockopt(sd, IPPROTO_IP, IP_TOS, (void*)&optval, sizeof(optval)) ==-1){
        ERROR("WARNING: setsockopt(tos): %s\n", strerror(errno));
	/* continue since this is not critical */
    }

    struct sockaddr_in sa;
    sa.sin_family = AF_INET;
    sa.sin_port = htons(udp_port);
    sa.sin_addr.s_addr = INADDR_ANY;

    bool socket_bound = false;
    while(!socket_bound){
	if( bind(sd,(sockaddr*)&sa,sizeof(struct sockaddr_in)) == -1 ){
	    ERROR("could not bind socket at port %i: %s\n",udp_port,strerror(errno));
	    udp_port += 2;
	    sa.sin_port = htons(udp_port);
	}
	else{
	    DBG("socket bound at port %i\n",udp_port);
	    socket_bound = true;
	}
    }
    struct sockaddr_in addr;
    socklen_t addrlen = sizeof(struct sockaddr_in);

    char msg_buf[MSG_BUF_SIZE];
    int  msg_buf_s;

    while(true){

	msg_buf_s = recvfrom(sd,msg_buf,MSG_BUF_SIZE,0,(sockaddr*)&addr,&addrlen);
	if(msg_buf_s == -1){

	    switch(errno){
		case EINTR:
		case EAGAIN:
		    continue;
		default: break;
	    };

	    ERROR("recvfrom: %s\n",strerror(errno));
	    break;
	}

	printf("received packet from: %s:%i\n",
	       inet_ntoa(addr.sin_addr),ntohs(addr.sin_port));

	string             reply;
	struct sockaddr_in reply_addr;

	if(execute(msg_buf,reply,reply_addr) == -1)
	    continue;

	send_reply(reply,addr);
    }
    
}



int StatsUDPServer::execute(char* msg_buf, string& reply, 
			    struct sockaddr_in& addr)
{
    string cmd_str,reply_addr,reply_port;
    char *msg_c = msg_buf;
    msg_get_param(msg_c,cmd_str);

    if(cmd_str == "calls")
	reply = "Active calls: " + int2str(sc->getSize()) + "\n";
    else
	reply = "Unknown command: '" + cmd_str + "'\n";

    return 0;
}

int StatsUDPServer::send_reply(const string& reply,
			       const struct sockaddr_in& reply_addr)
{
    int err = sendto(sd,reply.c_str(),reply.length()+1,0,
		     (const sockaddr*)&reply_addr,
		     sizeof(struct sockaddr_in));

    return (err <= 0) ? -1 : 0;
}
